import { FilterQuery, UpdateQuery } from 'mongoose';
import { AxiosHeaders } from 'axios';
import { UpdateResult, DeleteResult, Filter } from 'mongodb';

interface ObjectIdDto {
    _id: string;
}
interface FilterDto<T extends Record<string, any>> {
    filter: FilterQuery<T>;
}
interface DataSingleDto<T extends Record<string, any>> {
    data: T;
}
interface DataArrayDto<T extends Record<string, any>> {
    data: T[];
}
interface UpdateIdDto<T extends Record<string, any>> {
    _id: string;
    update: UpdateQuery<T>;
}
interface UpdateDto<T extends Record<string, any>> {
    filter: FilterQuery<T>;
    update: UpdateQuery<T>;
}
type Deserializers<T extends Record<string, any>> = {
    [K in keyof T]: (data: any) => T[K];
};
/**
 * Configuration options for the Realtime Client.
 *
 * @template ModelMap - A mapping of model names to their respective data types.
 */
interface RealtimeClientOptions<ModelMap extends Record<string, any> = Record<string, any>> {
    /**
     * The base URL of the API server.
     * This is the root URL where your API is hosted.
     *
     * @example 'https://api.example.com'
     */
    baseURL: string;
    /**
     * Custom headers to include with every HTTP request made by the client.
     * This should not be used for authentication headers.
     * Use this to set any static headers required by your API.
     *
     * @example
     * {
     *   'Content-Type': 'application/json',
     *   'X-Custom-Header': 'custom-value',
     * }
     */
    headers?: Record<string, string> | AxiosHeaders;
    /**
     * A function that returns an authentication payload for WebSocket connections.
     * This payload will be sent during the WebSocket handshake to authenticate the client.
     *
     * The function can return the payload directly or return a promise that resolves to the payload.
     *
     * @returns A promise that resolves to an object containing authentication data, or the object itself.
     *
     * @example
     * // Asynchronous function returning a promise
     * async () => {
     *   const token = await fetchWebSocketToken();
     *   return { token };
     * }
     */
    wsAuth?: () => Promise<Record<string, any>> | Record<string, any>;
    /**
     * A function that returns authentication data for REST API requests.
     * This function is called before each REST API request to retrieve fresh authentication data.
     *
     * The function can return the authentication data directly or return a promise that resolves to the data.
     *
     * @returns A promise that resolves to an object containing authentication data (e.g., headers), or the object itself.
     *
     * @example
     * // Asynchronous function returning a promise
     * async () => {
     *   const token = await fetchRestApiToken();
     *   return { 'Authorization': `Bearer ${token}` };
     * }
     */
    restAuth?: () => Promise<Record<string, string>> | Record<string, string>;
    /**
     * Indicates whether cross-site Access-Control requests should be made using credentials such as cookies, authorization headers, or TLS client certificates.
     *
     * Set this to `true` if your API requires credentials like cookies or you need to make authenticated cross-origin requests.
     *
     * @default false
     *
     * @example
     * true
     */
    withCredentials?: boolean;
    /**
     * An object containing functions to deserialize data received from the server.
     * These functions can be used to transform raw data into instances of classes or perform custom data transformations.
     *
     * Each key should correspond to a model name, and the value is a function that accepts raw data and returns the deserialized object.
     *
     * @example
     * {
     *   User: (data) => new UserModel(data),
     *   Post: (data) => new PostModel(data),
     * }
     */
    deserializers?: Deserializers<ModelMap>;
}

declare class RealtimeRestClient<ModelMap extends Record<string, any> = Record<string, any>, CreateModelExclusion extends Record<string, any> = {
    _id: string;
    __v: string;
}> {
    private readonly options;
    private axiosInstance;
    constructor(options: RealtimeClientOptions<ModelMap>);
    private isException;
    private deserialize;
    /**
     * Inserts a single document into the specified model in the database.
     *
     * @template ModelName - The name of the model into which the document will be inserted. It must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the data to be inserted.
     *   - `data`: The actual document to insert into the database.
     *
     * @returns A promise that resolves to the inserted document of type `ModelMap[ModelName]`.
     *
     * @example
     * ```typescript
     * // Assuming you have a 'User' model in your ModelMap
     * const newUser = await client.insertOne('User', {
     *   data: {
     *     username: 'john_doe',
     *     email: 'john@example.com',
     *     password: 'securePassword123',
     *   },
     * });
     * console.log('Inserted user:', newUser);
     * ```
     */
    insertOne: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: DataSingleDto<Omit<ModelMap[ModelName], keyof CreateModelExclusion>>) => Promise<ModelMap[ModelName]>;
    /**
     * Inserts multiple documents into the specified model in the database.
     *
     * @template ModelName - The name of the model into which the documents will be inserted. It must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing an array of data to be inserted.
     *   - `data`: An array of documents to insert into the database.
     *
     * @returns A promise that resolves to an array of inserted documents of type `ModelMap[ModelName][]`.
     *
     * @example
     * ```typescript
     * // Inserting multiple users into the 'User' model
     * const newUsers = await client.insertMany('User', {
     *   data: [
     *     { username: 'john_doe', email: 'john@example.com' },
     *     { username: 'jane_doe', email: 'jane@example.com' },
     *   ],
     * });
     * console.log('Inserted users:', newUsers);
     * ```
     */
    insertMany: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: DataArrayDto<Omit<ModelMap[ModelName], keyof CreateModelExclusion>>) => Promise<Array<ModelMap[ModelName]>>;
    /**
     * Finds a single document in the specified model that matches the given filter.
     *
     * @template ModelName - The name of the model to query; must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the filter criteria.
     *   - `filter`: A MongoDB query filter specifying the criteria to match.
     *
     * @returns A promise that resolves to the found document of type `ModelMap[ModelName]`, or `null` if no document matches.
     *
     * @example
     * ```typescript
     * // Finding a user with a specific username
     * const user = await client.findOne('User', {
     *   filter: { username: 'john_doe' },
     * });
     * if (user) {
     *   console.log('Found user:', user);
     * } else {
     *   console.log('User not found');
     * }
     * ```
     */
    findOne: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: FilterDto<ModelMap[ModelName]>) => Promise<ModelMap[ModelName] | null>;
    /**
     * Finds multiple documents in the specified model that match the given filter.
     *
     * @template ModelName - The name of the model to query; must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the filter criteria.
     *   - `filter`: A MongoDB query filter specifying the criteria to match.
     *
     * @returns A promise that resolves to an array of documents of type `ModelMap[ModelName]` that match the filter.
     *
     * @example
     * ```typescript
     * // Finding all users with the role 'admin'
     * const admins = await client.findMany('User', {
     *   filter: { role: 'admin' },
     * });
     * console.log('Admin users:', admins);
     * ```
     */
    findMany: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: FilterDto<ModelMap[ModelName]>) => Promise<Array<ModelMap[ModelName]>>;
    /**
     * Finds a single document in the specified model by its unique identifier.
     *
     * @template ModelName - The name of the model to query; must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the unique identifier of the document.
     *   - `_id`: The unique identifier (ObjectId) of the document to retrieve.
     *
     * @returns A promise that resolves to the found document of type `ModelMap[ModelName]`, or `null` if no document matches.
     *
     * @example
     * ```typescript
     * // Finding a user by their ID
     * const user = await client.findById('User', {
     *   _id: '60d0fe4f5311236168a109ca',
     * });
     * if (user) {
     *   console.log('Found user:', user);
     * } else {
     *   console.log('User not found');
     * }
     * ```
     */
    findById: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: ObjectIdDto) => Promise<ModelMap[ModelName] | null>;
    /**
     * Updates a single document in the specified model that matches the given filter.
     *
     * @template ModelName - The name of the model to update; must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the filter criteria and the update operations.
     *   - `filter`: A MongoDB query filter specifying which document to update.
     *   - `update`: A MongoDB update object defining the updates to apply.
     *
     * @returns A promise that resolves to an `UpdateResult` object containing information about the operation.
     *
     * @example
     * ```typescript
     * // Updating a user's email address
     * const result = await client.updateOne('User', {
     *   filter: { username: 'john_doe' },
     *   update: { $set: { email: 'new_email@example.com' } },
     * });
     * console.log('Number of documents matched:', result.matchedCount);
     * console.log('Number of documents modified:', result.modifiedCount);
     * ```
     */
    updateOne: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: UpdateDto<ModelMap[ModelName]>) => Promise<UpdateResult>;
    /**
     * Updates multiple documents in the specified model that match the given filter.
     *
     * @template ModelName - The name of the model to update; must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the filter criteria and the update operations.
     *   - `filter`: A MongoDB query filter specifying which documents to update.
     *   - `update`: A MongoDB update object defining the updates to apply.
     *
     * @returns A promise that resolves to an `UpdateResult` object containing information about the operation.
     *
     * @example
     * ```typescript
     * // Incrementing the login count for all active users
     * const result = await client.updateMany('User', {
     *   filter: { isActive: true },
     *   update: { $inc: { loginCount: 1 } },
     * });
     * console.log('Number of documents matched:', result.matchedCount);
     * console.log('Number of documents modified:', result.modifiedCount);
     * ```
     */
    updateMany: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: UpdateDto<ModelMap[ModelName]>) => Promise<UpdateResult>;
    /**
     * Finds a single document matching the filter, updates it, and returns the updated document.
     *
     * @template ModelName - The name of the model to update; must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the filter criteria and the update operations.
     *   - `filter`: A MongoDB query filter specifying which document to find and update.
     *   - `update`: A MongoDB update object defining the updates to apply.
     *
     * @returns A promise that resolves to the updated document of type `ModelMap[ModelName]`, or `null` if no document matches.
     *
     * @example
     * ```typescript
     * // Find a user by username and update their email, returning the updated document
     * const updatedUser = await client.findOneAndUpdate('User', {
     *   filter: { username: 'john_doe' },
     *   update: { $set: { email: 'new_email@example.com' } },
     * });
     * if (updatedUser) {
     *   console.log('Updated user:', updatedUser);
     * } else {
     *   console.log('User not found');
     * }
     * ```
     */
    findOneAndUpdate: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: UpdateDto<ModelMap[ModelName]>) => Promise<ModelMap[ModelName] | null>;
    /**
     * Finds a document by its unique identifier, updates it, and returns the updated document.
     *
     * @template ModelName - The name of the model to update; must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the unique identifier and the update operations.
     *   - `_id`: The unique identifier (ObjectId) of the document to update.
     *   - `update`: A MongoDB update object defining the updates to apply.
     *
     * @returns A promise that resolves to the updated document of type `ModelMap[ModelName]`, or `null` if no document matches.
     *
     * @example
     * ```typescript
     * // Find a user by ID and update their status, returning the updated document
     * const updatedUser = await client.findByIdAndUpdate('User', {
     *   _id: '60d0fe4f5311236168a109ca',
     *   update: { $set: { isActive: false } },
     * });
     * if (updatedUser) {
     *   console.log('Updated user:', updatedUser);
     * } else {
     *   console.log('User not found');
     * }
     * ```
     */
    findByIdAndUpdate: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: UpdateIdDto<ModelMap[ModelName]>) => Promise<ModelMap[ModelName] | null>;
    /**
     * Replaces a single document in the specified model that matches the given filter.
     *
     * @template ModelName - The name of the model to replace; must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the filter criteria and the replacement document.
     *   - `filter`: A MongoDB query filter specifying which document to replace.
     *   - `replace`: The new document that will replace the existing one.
     *
     * @returns A promise that resolves to an `UpdateResult` object containing information about the operation.
     *
     * @example
     * ```typescript
     * // Replacing a product document with new data
     * const result = await client.replaceOne('Product', {
     *   filter: { sku: 'ABC123' },
     *   replace: {
     *     sku: 'ABC123',
     *     name: 'New Product Name',
     *     price: 29.99,
     *     inStock: true,
     *   },
     * });
     * console.log('Number of documents matched:', result.matchedCount);
     * console.log('Number of documents modified:', result.modifiedCount);
     * ```
     */
    replaceOne: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: UpdateDto<ModelMap[ModelName]>) => Promise<UpdateResult>;
    /**
     * Finds a single document matching the filter, replaces it with a new document, and returns the replaced document.
     *
     * @template ModelName - The name of the model to replace; must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the filter criteria and the replacement document.
     *   - `filter`: A MongoDB query filter specifying which document to find and replace.
     *   - `replace`: The new document that will replace the existing one.
     *
     * @returns A promise that resolves to the replaced document of type `ModelMap[ModelName]`, or `null` if no document matches.
     *
     * @example
     * ```typescript
     * // Find a product by SKU and replace it, returning the old document
     * const oldProduct = await client.findOneAndReplace('Product', {
     *   filter: { sku: 'ABC123' },
     *   replace: {
     *     sku: 'ABC123',
     *     name: 'Updated Product Name',
     *     price: 39.99,
     *     inStock: false,
     *   },
     * });
     * if (oldProduct) {
     *   console.log('Replaced product:', oldProduct);
     * } else {
     *   console.log('Product not found');
     * }
     * ```
     */
    findOneAndReplace: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: UpdateDto<ModelMap[ModelName]>) => Promise<ModelMap[ModelName] | null>;
    /**
     * Deletes a single document in the specified model that matches the given filter criteria.
     *
     * @template ModelName - The name of the model from which to delete the document; must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the filter criteria.
     *   - `filter`: A MongoDB query filter specifying which document to delete.
     *
     * @returns A promise that resolves to a `DeleteResult` object containing information about the operation.
     *
     * @example
     * ```typescript
     * // Deleting a user with a specific username
     * const result = await client.deleteOne('User', {
     *   filter: { username: 'john_doe' },
     * });
     * console.log('Number of documents deleted:', result.deletedCount);
     * ```
     */
    deleteOne: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: FilterDto<ModelMap[ModelName]>) => Promise<DeleteResult>;
    /**
     * Deletes multiple documents in the specified model that match the given filter criteria.
     *
     * @template ModelName - The name of the model from which to delete the documents; must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the filter criteria.
     *   - `filter`: A MongoDB query filter specifying which documents to delete.
     *
     * @returns A promise that resolves to a `DeleteResult` object containing information about the operation.
     *
     * @example
     * ```typescript
     * // Deleting all inactive users
     * const result = await client.deleteMany('User', {
     *   filter: { isActive: false },
     * });
     * console.log('Number of documents deleted:', result.deletedCount);
     * ```
     */
    deleteMany: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: FilterDto<ModelMap[ModelName]>) => Promise<DeleteResult>;
    /**
     * Finds a single document matching the filter, deletes it, and returns the deleted document.
     *
     * @template ModelName - The name of the model from which to delete the document; must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the filter criteria.
     *   - `filter`: A MongoDB query filter specifying which document to find and delete.
     *
     * @returns A promise that resolves to the deleted document of type `ModelMap[ModelName]`, or `null` if no document matches.
     *
     * @example
     * ```typescript
     * // Find and delete a user by username, returning the deleted document
     * const deletedUser = await client.findOneAndDelete('User', {
     *   filter: { username: 'john_doe' },
     * });
     * if (deletedUser) {
     *   console.log('Deleted user:', deletedUser);
     * } else {
     *   console.log('User not found');
     * }
     * ```
     */
    findOneAndDelete: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: FilterDto<ModelMap[ModelName]>) => Promise<ModelMap[ModelName] | null>;
    /**
     * Finds a document by its unique identifier, deletes it, and returns the deleted document.
     *
     * @template ModelName - The name of the model from which to delete the document; must be a key of `ModelMap`.
     *
     * @param modelName - The name of the model as defined in your `ModelMap`.
     * @param payload - An object containing the unique identifier of the document.
     *   - `_id`: The unique identifier (ObjectId) of the document to delete.
     *
     * @returns A promise that resolves to the deleted document of type `ModelMap[ModelName]`, or `null` if no document matches.
     *
     * @example
     * ```typescript
     * // Find and delete a user by ID, returning the deleted document
     * const deletedUser = await client.findByIdAndDelete('User', {
     *   _id: '60d0fe4f5311236168a109ca',
     * });
     * if (deletedUser) {
     *   console.log('Deleted user:', deletedUser);
     * } else {
     *   console.log('User not found');
     * }
     * ```
     */
    findByIdAndDelete: <ModelName extends keyof ModelMap>(modelName: ModelName, payload: ObjectIdDto) => Promise<ModelMap[ModelName] | null>;
}

interface RealtimeResult {
    unsubscribe: () => void;
}
declare class RealtimeSocketClient<ModelMap extends Record<string, any> = Record<string, any>> {
    private readonly options;
    constructor(options: RealtimeClientOptions<ModelMap>);
    private deserialize;
    private _subscribe;
    onQuery: <ModelName extends keyof ModelMap, R extends ModelMap[ModelName]>(modelName: ModelName, filter: Filter<R>, onChange: (data: R[]) => any, onError?: (err: any) => any) => RealtimeResult;
    onDocument: <ModelName extends keyof ModelMap, R extends ModelMap[ModelName]>(modelName: ModelName, _id: string, onChange: (data: R | null) => any, onError?: (err: any) => any) => RealtimeResult;
}

declare class ApiError extends Error {
    statusCode: number;
    details?: any;
    constructor(message: string, statusCode: number, details?: any);
}
declare class UnauthorizedError extends ApiError {
    constructor(message: string, status: number, details?: any);
}
declare class ValidationError extends ApiError {
    constructor(message: string, details: any);
}
declare class ConflictError extends ApiError {
    constructor(message: string, details?: any);
}
declare class ServiceUnavailableError extends ApiError {
    constructor(message: string, details?: any);
}
declare class InternalServerError extends ApiError {
    constructor(message: string, details?: any);
}

declare const initializeRealtimeMongo: <ModelMap extends Record<string, any> = Record<string, any>, CreateModelExclusion extends Record<string, any> = {
    _id: string;
    __v: string;
}>(options: RealtimeClientOptions<ModelMap>) => {
    databaseRest: RealtimeRestClient<ModelMap, CreateModelExclusion>;
    databaseSocket: RealtimeSocketClient<ModelMap>;
};

export { ApiError, ConflictError, InternalServerError, type RealtimeClientOptions, RealtimeRestClient, RealtimeSocketClient, ServiceUnavailableError, UnauthorizedError, ValidationError, initializeRealtimeMongo };
